#!/usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ws_1 = require("ws");
const commander_1 = require("commander");
const c = __importStar(require("ansi-colors"));
const tslog_1 = require("tslog");
const readline = __importStar(require("readline"));
const events_1 = require("events");
const date_and_time_1 = __importDefault(require("date-and-time"));
const const_1 = require("../lib/const");
const command_1 = require("../lib/driver/command");
const command_2 = require("../lib/device/command");
const command_3 = require("../lib/station/command");
const utils_1 = require("../lib/utils");
const commands = Object.values(command_1.DriverCommand).concat(Object.values(command_2.DeviceCommand)).concat(Object.values(command_3.StationCommand)).concat(["quit", "exit"]);
const devicePropertiesMetadata = {};
const stationPropertiesMetadata = {};
const emitter = new events_1.EventEmitter();
const parsePropertyValue = (property, value, serialNumber, propertiesMetadata) => {
    try {
        if (propertiesMetadata[serialNumber] !== undefined && propertiesMetadata[serialNumber][property] !== undefined) {
            switch (propertiesMetadata[serialNumber][property].type) {
                case "boolean":
                    return value.toLowerCase() === "true" ? true : false;
                case "number":
                    return Number.parseInt(value);
                case "object":
                    return JSON.parse(value);
                case "string":
                default:
                    return value;
            }
        }
    }
    catch (error) {
        console.log("ERROR", error);
    }
    return value;
};
const cmdHelp = (cmd) => {
    switch (cmd) {
        case command_1.DriverCommand.setVerifyCode:
            console.log(`${cmd} <numeric_code>`);
            break;
        case command_1.DriverCommand.setCaptcha:
            console.log(`${cmd} <captcha> [<id>]`);
            break;
        case command_1.DriverCommand.pollRefresh:
        case command_1.DriverCommand.isConnected:
        case command_1.DriverCommand.isPushConnected:
        case command_1.DriverCommand.connect:
        case command_1.DriverCommand.disconnect:
        case command_1.DriverCommand.getVideoEvents:
        case command_1.DriverCommand.getAlarmEvents:
        case command_1.DriverCommand.getHistoryEvents:
        case command_1.DriverCommand.getLogLevel:
        case command_1.DriverCommand.startListeningLogs:
        case command_1.DriverCommand.stopListeningLogs:
            console.log(`${cmd}`);
            break;
        case command_1.DriverCommand.setLogLevel:
            console.log(`${cmd} <loglevel>`);
            break;
        case command_2.DeviceCommand.setStatusLed:
        case command_2.DeviceCommand.setAutoNightVision:
        case command_2.DeviceCommand.setMotionDetection:
        case command_2.DeviceCommand.setSoundDetection:
        case command_2.DeviceCommand.setPetDetection:
        case command_2.DeviceCommand.setRTSPStream:
        case command_2.DeviceCommand.setAntiTheftDetection:
        case command_2.DeviceCommand.enableDevice:
        case command_2.DeviceCommand.lockDevice:
            console.log(`${cmd} <device_sn> <true|false>`);
            break;
        case command_2.DeviceCommand.setWatermark:
            console.log(`${cmd} <device_sn> <numeric_option>`);
            break;
        case command_2.DeviceCommand.getPropertiesMetadata:
        case command_2.DeviceCommand.getProperties:
        case command_2.DeviceCommand.startLivestream:
        case command_2.DeviceCommand.stopLivestream:
        case command_2.DeviceCommand.resetAlarm:
        case command_2.DeviceCommand.getVoices:
        case command_2.DeviceCommand.cancelDownload:
        case command_2.DeviceCommand.isLiveStreaming:
        case command_2.DeviceCommand.getCommands:
        case command_2.DeviceCommand.startRTSPLivestream:
        case command_2.DeviceCommand.stopRTSPLivestream:
        case command_2.DeviceCommand.isRTSPLiveStreaming:
        case command_2.DeviceCommand.calibrateLock:
        case command_2.DeviceCommand.calibrate:
        case command_2.DeviceCommand.setDefaultAngle:
        case command_2.DeviceCommand.setPrivacyAngle:
        case command_2.DeviceCommand.unlock:
        case command_2.DeviceCommand.startTalkback:
        case command_2.DeviceCommand.stopTalkback:
        case command_2.DeviceCommand.isTalkbackOngoing:
        case command_2.DeviceCommand.isDownloading:
        case command_2.DeviceCommand.getUsers:
            console.log(`${cmd} <device_sn>`);
            break;
        case command_2.DeviceCommand.verifyPIN:
            console.log(`${cmd} <device_sn> <pin>`);
            break;
        case command_2.DeviceCommand.setProperty:
            console.log(`${cmd} <device_sn> <name> <value>|<json_string>`);
            break;
        case command_2.DeviceCommand.triggerAlarm:
            console.log(`${cmd} <device_sn> <seconds>`);
            break;
        case command_2.DeviceCommand.panAndTilt:
            console.log(`${cmd} <device_sn> <direction>`);
            break;
        case command_2.DeviceCommand.quickResponse:
            console.log(`${cmd} <device_sn> <voiceId>`);
            break;
        case command_2.DeviceCommand.startDownload:
            console.log(`${cmd} <device_sn> <path> [cipherId]`);
            break;
        case command_2.DeviceCommand.hasProperty:
        case command_2.DeviceCommand.hasCommand:
            console.log(`${cmd} <device_sn> <name>`);
            break;
        case command_2.DeviceCommand.snooze:
            console.log(`${cmd} <device_sn> <snooze_time> [snooze_chime] [snooze_motion] [snooze_homebase]`);
            break;
        case command_2.DeviceCommand.addUser:
            console.log(`${cmd} <device_sn> <username> <passcode> [schedule]`);
        case command_2.DeviceCommand.deleteUser:
            console.log(`${cmd} <device_sn> <username>`);
        case command_2.DeviceCommand.updateUser:
            console.log(`${cmd} <device_sn> <username> <new_username>`);
        case command_2.DeviceCommand.updateUserSchedule:
            console.log(`${cmd} <device_sn> <username> <schedule>`);
        case command_2.DeviceCommand.updateUserPasscode:
            console.log(`${cmd} <device_sn> <username> <passcode>`);
        case command_3.StationCommand.setGuardMode:
            console.log(`${cmd} <station_sn> <numeric_code>`);
            break;
        case command_3.StationCommand.isConnected:
        case command_3.StationCommand.connect:
        case command_3.StationCommand.disconnect:
        case command_3.StationCommand.resetAlarm:
        case command_3.StationCommand.reboot:
        case command_3.StationCommand.getCommands:
            console.log(`${cmd} <station_sn>`);
            break;
        case command_3.StationCommand.getPropertiesMetadata:
        case command_3.StationCommand.getProperties:
            console.log(`${cmd} <station_sn>`);
            break;
        case command_3.StationCommand.setProperty:
            console.log(`${cmd} <station_sn> <name> <value>`);
            break;
        case command_3.StationCommand.triggerAlarm:
            console.log(`${cmd} <station_sn> <seconds>`);
            break;
        case command_3.StationCommand.hasProperty:
        case command_3.StationCommand.hasCommand:
            console.log(`${cmd} <station_sn> <name>`);
            break;
        case command_3.StationCommand.chime:
            console.log(`${cmd} <station_sn> [ringtone]`);
            break;
        case command_3.StationCommand.downloadImage:
            console.log(`${cmd} <station_sn> <file>`);
            break;
        case command_3.StationCommand.databaseQueryLatestInfo:
            console.log(`${cmd} <station_sn>`);
            break;
        case command_3.StationCommand.databaseQueryLocal:
            console.log(`${cmd} <station_sn> <device_sn1,device_sn2,...> <startDate (YYYYMMDD)> <endDate (YYYYMMDD)> [eventType] [detectionType] [storageType]`);
            break;
        case command_3.StationCommand.databaseCountByDate:
            console.log(`${cmd} <station_sn> <startDate (YYYYMMDD)> <endDate (YYYYMMDD)>`);
            break;
        case command_3.StationCommand.databaseDelete:
            console.log(`${cmd} <station_sn> <id1> [<id2> ...]`);
            break;
        default:
            console.log(`Type HELP "command name" to display more information about a specific command.`);
            Object.values(command_1.DriverCommand).forEach((cmd) => {
                console.log(cmd);
            });
            Object.values(command_2.DeviceCommand).forEach((cmd) => {
                console.log(cmd);
            });
            Object.values(command_3.StationCommand).forEach((cmd) => {
                console.log(cmd);
            });
            console.log("exit");
            console.log("quit");
            break;
    }
};
const cmd = async (args, silent = false, internal = false) => {
    switch (args[0]) {
        case "help":
            if (args.length <= 1 || args.length > 2) {
                cmdHelp("");
            }
            else {
                cmdHelp(args[1]);
            }
            break;
        case "exit":
        case "quit":
            break;
        case command_1.DriverCommand.setVerifyCode:
            if (args.length === 2 && isNumber(args[1])) {
                socket.send(JSON.stringify({
                    messageId: command_1.DriverCommand.setVerifyCode.split(".")[1],
                    command: command_1.DriverCommand.setVerifyCode,
                    verifyCode: args[1]
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_1.DriverCommand.setCaptcha:
            if (args.length === 2 || args.length === 3) {
                socket.send(JSON.stringify({
                    messageId: command_1.DriverCommand.setCaptcha.split(".")[1],
                    command: command_1.DriverCommand.setCaptcha,
                    captchaId: args.length === 3 ? args[2] : undefined,
                    captcha: args[1]
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_1.DriverCommand.pollRefresh:
            if (args.length === 1) {
                socket.send(JSON.stringify({
                    messageId: command_1.DriverCommand.pollRefresh.split(".")[1],
                    command: command_1.DriverCommand.pollRefresh
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_1.DriverCommand.isConnected:
            if (args.length === 1) {
                socket.send(JSON.stringify({
                    messageId: command_1.DriverCommand.isConnected.split(".")[1],
                    command: command_1.DriverCommand.isConnected
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_1.DriverCommand.isPushConnected:
            if (args.length === 1) {
                socket.send(JSON.stringify({
                    messageId: command_1.DriverCommand.isPushConnected.split(".")[1],
                    command: command_1.DriverCommand.isPushConnected
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_1.DriverCommand.connect:
            if (args.length === 1) {
                socket.send(JSON.stringify({
                    messageId: command_1.DriverCommand.connect.split(".")[1],
                    command: command_1.DriverCommand.connect
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_1.DriverCommand.disconnect:
            if (args.length === 1) {
                socket.send(JSON.stringify({
                    messageId: command_1.DriverCommand.disconnect.split(".")[1],
                    command: command_1.DriverCommand.disconnect
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_1.DriverCommand.getVideoEvents:
            if (args.length === 1) {
                socket.send(JSON.stringify({
                    messageId: command_1.DriverCommand.getVideoEvents.split(".")[1],
                    command: command_1.DriverCommand.getVideoEvents
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_1.DriverCommand.getAlarmEvents:
            if (args.length === 1) {
                socket.send(JSON.stringify({
                    messageId: command_1.DriverCommand.getAlarmEvents.split(".")[1],
                    command: command_1.DriverCommand.getAlarmEvents
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_1.DriverCommand.getHistoryEvents:
            if (args.length === 1) {
                socket.send(JSON.stringify({
                    messageId: command_1.DriverCommand.getHistoryEvents.split(".")[1],
                    command: command_1.DriverCommand.getHistoryEvents
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_1.DriverCommand.getLogLevel:
            if (args.length === 1) {
                socket.send(JSON.stringify({
                    messageId: command_1.DriverCommand.getLogLevel.split(".")[1],
                    command: command_1.DriverCommand.getLogLevel
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_1.DriverCommand.startListeningLogs:
            if (args.length === 1) {
                socket.send(JSON.stringify({
                    messageId: command_1.DriverCommand.startListeningLogs.split(".")[1],
                    command: command_1.DriverCommand.startListeningLogs
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_1.DriverCommand.stopListeningLogs:
            if (args.length === 1) {
                socket.send(JSON.stringify({
                    messageId: command_1.DriverCommand.stopListeningLogs.split(".")[1],
                    command: command_1.DriverCommand.stopListeningLogs
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_1.DriverCommand.setLogLevel:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_1.DriverCommand.setLogLevel.split(".")[1],
                    command: command_1.DriverCommand.setLogLevel,
                    level: args[1]
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.setStatusLed:
            if (args.length === 3 && isTrueFalse(args[2])) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.setStatusLed.split(".")[1],
                    command: command_2.DeviceCommand.setStatusLed,
                    serialNumber: args[1],
                    value: args[2].toLowerCase() === "true" ? true : false,
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.setAutoNightVision:
            if (args.length === 3 && isTrueFalse(args[2])) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.setAutoNightVision.split(".")[1],
                    command: command_2.DeviceCommand.setAutoNightVision,
                    serialNumber: args[1],
                    value: args[2].toLowerCase() === "true" ? true : false,
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.setMotionDetection:
            if (args.length === 3 && isTrueFalse(args[2])) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.setMotionDetection.split(".")[1],
                    command: command_2.DeviceCommand.setMotionDetection,
                    serialNumber: args[1],
                    value: args[2].toLowerCase() === "true" ? true : false,
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.setSoundDetection:
            if (args.length === 3 && isTrueFalse(args[2])) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.setSoundDetection.split(".")[1],
                    command: command_2.DeviceCommand.setSoundDetection,
                    serialNumber: args[1],
                    value: args[2].toLowerCase() === "true" ? true : false,
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.setPetDetection:
            if (args.length === 3 && isTrueFalse(args[2])) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.setPetDetection.split(".")[1],
                    command: command_2.DeviceCommand.setPetDetection,
                    serialNumber: args[1],
                    value: args[2].toLowerCase() === "true" ? true : false,
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.setRTSPStream:
            if (args.length === 3 && isTrueFalse(args[2])) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.setRTSPStream.split(".")[1],
                    command: command_2.DeviceCommand.setRTSPStream,
                    serialNumber: args[1],
                    value: args[2].toLowerCase() === "true" ? true : false,
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.setAntiTheftDetection:
            if (args.length === 3 && isTrueFalse(args[2])) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.setAntiTheftDetection.split(".")[1],
                    command: command_2.DeviceCommand.setAntiTheftDetection,
                    serialNumber: args[1],
                    value: args[2].toLowerCase() === "true" ? true : false,
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.enableDevice:
            if (args.length === 3 && isTrueFalse(args[2])) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.enableDevice.split(".")[1],
                    command: command_2.DeviceCommand.enableDevice,
                    serialNumber: args[1],
                    value: args[2].toLowerCase() === "true" ? true : false,
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.lockDevice:
            if (args.length === 3 && isTrueFalse(args[2])) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.lockDevice.split(".")[1],
                    command: command_2.DeviceCommand.lockDevice,
                    serialNumber: args[1],
                    value: args[2].toLowerCase() === "true" ? true : false,
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.setWatermark:
            if (args.length === 3 && isNumber(args[2])) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.setWatermark.split(".")[1],
                    command: command_2.DeviceCommand.setWatermark,
                    serialNumber: args[1],
                    value: Number.parseInt(args[2]),
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.getPropertiesMetadata:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: internal ? `internal_${command_2.DeviceCommand.getPropertiesMetadata}` : command_2.DeviceCommand.getPropertiesMetadata.split(".")[1],
                    command: command_2.DeviceCommand.getPropertiesMetadata,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.getProperties:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.getProperties.split(".")[1],
                    command: command_2.DeviceCommand.getProperties,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.setProperty:
            if (args.length === 4) {
                if (devicePropertiesMetadata[args[1]] === undefined) {
                    await cmd(["device.get_properties_metadata", args[1]], false, true);
                    await (0, utils_1.waitForEvent)(emitter, `device.get_properties_metadata.${args[1]}`, 10000).catch(); //TODO: Handle better the timeout exception
                }
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.setProperty.split(".")[1],
                    command: command_2.DeviceCommand.setProperty,
                    serialNumber: args[1],
                    name: args[2],
                    value: parsePropertyValue(args[2], args[3], args[1], devicePropertiesMetadata),
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.startLivestream:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.startLivestream.split(".")[1],
                    command: command_2.DeviceCommand.startLivestream,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.stopLivestream:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.stopLivestream.split(".")[1],
                    command: command_2.DeviceCommand.stopLivestream,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.isLiveStreaming:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.isLiveStreaming.split(".")[1],
                    command: command_2.DeviceCommand.isLiveStreaming,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.triggerAlarm:
            if (args.length === 3) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.triggerAlarm.split(".")[1],
                    command: command_2.DeviceCommand.triggerAlarm,
                    serialNumber: args[1],
                    seconds: Number.parseInt(args[2]),
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.panAndTilt:
            if (args.length === 3) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.panAndTilt.split(".")[1],
                    command: command_2.DeviceCommand.panAndTilt,
                    serialNumber: args[1],
                    direction: Number.parseInt(args[2]),
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.quickResponse:
            if (args.length === 3) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.quickResponse.split(".")[1],
                    command: command_2.DeviceCommand.quickResponse,
                    serialNumber: args[1],
                    voiceId: Number.parseInt(args[2]),
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.resetAlarm:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.resetAlarm.split(".")[1],
                    command: command_2.DeviceCommand.resetAlarm,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.getVoices:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.getVoices.split(".")[1],
                    command: command_2.DeviceCommand.getVoices,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.startDownload:
            if ((args.length === 3) ||
                (args.length === 4 && isNumber(args[3]))) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.startDownload.split(".")[1],
                    command: command_2.DeviceCommand.startDownload,
                    serialNumber: args[1],
                    path: args[2],
                    cipherId: args.length === 4 ? Number.parseInt(args[3]) : undefined,
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.cancelDownload:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.cancelDownload.split(".")[1],
                    command: command_2.DeviceCommand.cancelDownload,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.hasProperty:
            if (args.length === 3) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.hasProperty.split(".")[1],
                    command: command_2.DeviceCommand.hasProperty,
                    serialNumber: args[1],
                    propertyName: args[2],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.getCommands:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.getCommands.split(".")[1],
                    command: command_2.DeviceCommand.getCommands,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.hasCommand:
            if (args.length === 3) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.hasCommand.split(".")[1],
                    command: command_2.DeviceCommand.hasCommand,
                    serialNumber: args[1],
                    commandName: args[2],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.startRTSPLivestream:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.startRTSPLivestream.split(".")[1],
                    command: command_2.DeviceCommand.startRTSPLivestream,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.stopRTSPLivestream:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.stopRTSPLivestream.split(".")[1],
                    command: command_2.DeviceCommand.stopRTSPLivestream,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.isRTSPLiveStreaming:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.isRTSPLiveStreaming.split(".")[1],
                    command: command_2.DeviceCommand.isRTSPLiveStreaming,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.calibrateLock:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.calibrateLock.split(".")[1],
                    command: command_2.DeviceCommand.calibrateLock,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.calibrate:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.calibrate.split(".")[1],
                    command: command_2.DeviceCommand.calibrate,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.setDefaultAngle:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.setDefaultAngle.split(".")[1],
                    command: command_2.DeviceCommand.setDefaultAngle,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.setPrivacyAngle:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.setPrivacyAngle.split(".")[1],
                    command: command_2.DeviceCommand.setPrivacyAngle,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.unlock:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.unlock.split(".")[1],
                    command: command_2.DeviceCommand.unlock,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.startTalkback:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.startTalkback.split(".")[1],
                    command: command_2.DeviceCommand.startTalkback,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.stopTalkback:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.stopTalkback.split(".")[1],
                    command: command_2.DeviceCommand.stopTalkback,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.isTalkbackOngoing:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.isTalkbackOngoing.split(".")[1],
                    command: command_2.DeviceCommand.isTalkbackOngoing,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.isDownloading:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.isDownloading.split(".")[1],
                    command: command_2.DeviceCommand.isDownloading,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.snooze:
            if ((args.length === 3 && isNumber(args[2])) ||
                (args.length === 4 && isNumber(args[2]) && isTrueFalse(args[3])) ||
                (args.length === 5 && isNumber(args[2]) && isTrueFalse(args[3]) && isTrueFalse(args[4])) ||
                (args.length === 6 && isNumber(args[2]) && isTrueFalse(args[3]) && isTrueFalse(args[4]) && isTrueFalse(args[5]))) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.snooze.split(".")[1],
                    command: command_2.DeviceCommand.snooze,
                    serialNumber: args[1],
                    snoozeTime: Number.parseInt(args[2]),
                    snoozeChime: args.length >= 4 && args[3].toLowerCase() === "true" ? true : false,
                    snoozeMotion: args.length >= 5 && args[4].toLowerCase() === "true" ? true : false,
                    snoozeHomebase: args.length === 6 && args[5].toLowerCase() === "true" ? true : false,
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.addUser:
            if (args.length === 4 || args.length === 5) {
                const command = {
                    messageId: command_2.DeviceCommand.addUser.split(".")[1],
                    command: command_2.DeviceCommand.addUser,
                    serialNumber: args[1],
                    username: args[2],
                    passcode: args[3]
                };
                if (args.length === 5) {
                    //TODO: Correct type casting etc.
                    command.schedule = args[5];
                }
                socket.send(JSON.stringify(command));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.deleteUser:
            if (args.length === 3) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.deleteUser.split(".")[1],
                    command: command_2.DeviceCommand.deleteUser,
                    serialNumber: args[1],
                    username: args[2],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.getUsers:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.getUsers.split(".")[1],
                    command: command_2.DeviceCommand.getUsers,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.updateUser:
            if (args.length === 4) {
                const command = {
                    messageId: command_2.DeviceCommand.updateUser.split(".")[1],
                    command: command_2.DeviceCommand.updateUser,
                    serialNumber: args[1],
                    username: args[2],
                    newUsername: args[3]
                };
                socket.send(JSON.stringify(command));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.updateUserPasscode:
            if (args.length === 4 && isNumber(args[3])) {
                const command = {
                    messageId: command_2.DeviceCommand.updateUserPasscode.split(".")[1],
                    command: command_2.DeviceCommand.updateUserPasscode,
                    serialNumber: args[1],
                    username: args[2],
                    passcode: args[3]
                };
                socket.send(JSON.stringify(command));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.updateUserSchedule:
            if (args.length === 4) {
                const command = {
                    messageId: command_2.DeviceCommand.updateUserSchedule.split(".")[1],
                    command: command_2.DeviceCommand.updateUserSchedule,
                    serialNumber: args[1],
                    username: args[2],
                    schedule: args[3], //TODO: Correct type casting etc.
                };
                socket.send(JSON.stringify(command));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_2.DeviceCommand.verifyPIN:
            if (args.length === 3) {
                socket.send(JSON.stringify({
                    messageId: command_2.DeviceCommand.verifyPIN.split(".")[1],
                    command: command_2.DeviceCommand.verifyPIN,
                    serialNumber: args[1],
                    pin: args[2],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_3.StationCommand.setGuardMode:
            if (args.length === 3 && isNumber(args[2])) {
                socket.send(JSON.stringify({
                    messageId: command_3.StationCommand.setGuardMode.split(".")[1],
                    command: command_3.StationCommand.setGuardMode,
                    serialNumber: args[1],
                    mode: Number.parseInt(args[2]),
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_3.StationCommand.reboot:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_3.StationCommand.reboot.split(".")[1],
                    command: command_3.StationCommand.reboot,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_3.StationCommand.isConnected:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_3.StationCommand.isConnected.split(".")[1],
                    command: command_3.StationCommand.isConnected,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_3.StationCommand.connect:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_3.StationCommand.connect.split(".")[1],
                    command: command_3.StationCommand.connect,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_3.StationCommand.disconnect:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_3.StationCommand.disconnect.split(".")[1],
                    command: command_3.StationCommand.disconnect,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_3.StationCommand.getPropertiesMetadata:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: internal ? `internal_${command_3.StationCommand.getPropertiesMetadata}` : command_3.StationCommand.getPropertiesMetadata.split(".")[1],
                    command: command_3.StationCommand.getPropertiesMetadata,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_3.StationCommand.getProperties:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_3.StationCommand.getProperties.split(".")[1],
                    command: command_3.StationCommand.getProperties,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_3.StationCommand.setProperty:
            if (args.length === 4) {
                if (stationPropertiesMetadata[args[1]] === undefined) {
                    await cmd(["station.get_properties_metadata", args[1]], false, true);
                    await (0, utils_1.waitForEvent)(emitter, `station.get_properties_metadata.${args[1]}`, 10000).catch(); //TODO: Handle better the timeout exception
                }
                socket.send(JSON.stringify({
                    messageId: command_3.StationCommand.setProperty.split(".")[1],
                    command: command_3.StationCommand.setProperty,
                    serialNumber: args[1],
                    name: args[2],
                    value: parsePropertyValue(args[2], args[3], args[1], stationPropertiesMetadata),
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_3.StationCommand.triggerAlarm:
            if (args.length === 3) {
                socket.send(JSON.stringify({
                    messageId: command_3.StationCommand.triggerAlarm.split(".")[1],
                    command: command_3.StationCommand.triggerAlarm,
                    serialNumber: args[1],
                    seconds: Number.parseInt(args[2]),
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_3.StationCommand.resetAlarm:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_3.StationCommand.resetAlarm.split(".")[1],
                    command: command_3.StationCommand.resetAlarm,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_3.StationCommand.getCommands:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_3.StationCommand.getCommands.split(".")[1],
                    command: command_3.StationCommand.getCommands,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_3.StationCommand.hasProperty:
            if (args.length === 3) {
                socket.send(JSON.stringify({
                    messageId: command_3.StationCommand.hasProperty.split(".")[1],
                    command: command_3.StationCommand.hasProperty,
                    serialNumber: args[1],
                    propertyName: args[2],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_3.StationCommand.hasCommand:
            if (args.length === 3) {
                socket.send(JSON.stringify({
                    messageId: command_3.StationCommand.hasCommand.split(".")[1],
                    command: command_3.StationCommand.hasCommand,
                    serialNumber: args[1],
                    commandName: args[2],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_3.StationCommand.chime:
            if (args.length === 2 || (args.length === 3 && isNumber(args[2]))) {
                const command = {
                    messageId: command_3.StationCommand.chime.split(".")[1],
                    command: command_3.StationCommand.chime,
                    serialNumber: args[1],
                };
                if (args.length === 3) {
                    command.ringtone = Number.parseInt(args[2]);
                }
                socket.send(JSON.stringify(command));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_3.StationCommand.downloadImage:
            if (args.length === 3) {
                const command = {
                    messageId: command_3.StationCommand.downloadImage.split(".")[1],
                    command: command_3.StationCommand.downloadImage,
                    serialNumber: args[1],
                    file: args[2]
                };
                socket.send(JSON.stringify(command));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_3.StationCommand.databaseQueryLatestInfo:
            if (args.length === 2) {
                socket.send(JSON.stringify({
                    messageId: command_3.StationCommand.databaseQueryLatestInfo.split(".")[1],
                    command: command_3.StationCommand.databaseQueryLatestInfo,
                    serialNumber: args[1],
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_3.StationCommand.databaseQueryLocal:
            if ((args.length === 5 && isDate(args[3]) && isDate(args[4])) ||
                (args.length === 6 && isDate(args[3]) && isDate(args[4]) && isNumber(args[5])) ||
                (args.length === 7 && isDate(args[3]) && isDate(args[4]) && isNumber(args[5]) && isNumber(args[6])) ||
                (args.length === 8 && isDate(args[3]) && isDate(args[4]) && isNumber(args[5]) && isNumber(args[6]) && isNumber(args[7]))) {
                const command = {
                    messageId: command_3.StationCommand.databaseQueryLocal.split(".")[1],
                    command: command_3.StationCommand.databaseQueryLocal,
                    serialNumber: args[1],
                    serialNumbers: args[2].split(","),
                    startDate: args[3],
                    endDate: args[4]
                };
                if (args.length === 6) {
                    command.eventType = Number.parseInt(args[5]);
                }
                if (args.length === 7) {
                    command.detectionType = Number.parseInt(args[6]);
                }
                if (args.length === 8) {
                    command.storageType = Number.parseInt(args[7]);
                }
                socket.send(JSON.stringify(command));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_3.StationCommand.databaseCountByDate:
            if (args.length === 4 && isDate(args[2]) && isDate(args[3])) {
                socket.send(JSON.stringify({
                    messageId: command_3.StationCommand.databaseCountByDate.split(".")[1],
                    command: command_3.StationCommand.databaseCountByDate,
                    serialNumber: args[1],
                    startDate: args[2],
                    endDate: args[3]
                }));
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        case command_3.StationCommand.databaseDelete:
            if (args.length >= 3) {
                let argsOk = true;
                const ids = [];
                for (const arg of args.slice(2)) {
                    if (!isNumber(arg)) {
                        argsOk = false;
                        break;
                    }
                    else {
                        ids.push(Number.parseInt(arg));
                    }
                }
                if (argsOk) {
                    socket.send(JSON.stringify({
                        messageId: command_3.StationCommand.databaseDelete.split(".")[1],
                        command: command_3.StationCommand.databaseDelete,
                        serialNumber: args[1],
                        ids: ids
                    }));
                }
                else {
                    cmdHelp(args[0]);
                    if (silent)
                        handleShutdown(1);
                }
            }
            else {
                cmdHelp(args[0]);
                if (silent)
                    handleShutdown(1);
            }
            break;
        default:
            if (args[0] !== "")
                cmdHelp("");
            if (silent)
                handleShutdown(1);
            break;
    }
    return args;
};
const isNumber = (value) => {
    try {
        Number.parseInt(value);
        return true;
    }
    catch (error) {
        return false;
    }
};
const isTrueFalse = (value) => {
    if (value.toLowerCase() === "true" || value.toLowerCase() === "false") {
        return true;
    }
    return false;
};
const isDate = (value) => {
    return !isNaN(date_and_time_1.default.parse(value, "YYYYMMDD").getTime());
};
const program = new commander_1.Command();
program
    .addOption(new commander_1.Option("-s, --schemaVersion <host>", "Schema version the server should support").default(const_1.maxSchemaVersion, "max client supported version"))
    .addOption(new commander_1.Option("-H, --host <host>", "Host to connect to").default("localhost"))
    .addOption(new commander_1.Option("-p, --port <port>", "Port to connect to").default(3000))
    .addOption(new commander_1.Option("-c, --command <command_name>", "Silent command to execute").choices(commands))
    .addOption(new commander_1.Option("-a, --arguments <args...>", "Arguments for silent command if expected"))
    .addOption(new commander_1.Option("-t, --timeout <seconds>", "Silent command timeout seconds").default(30))
    .addOption(new commander_1.Option("-v, --verbose"));
program.parse(process.argv);
const options = program.opts();
const schemaVersion = options.schemaVersion
    ? Number(options.schemaVersion)
    : const_1.maxSchemaVersion;
const url = `ws://${options.host}:${options.port}`;
if (isNaN(schemaVersion) || schemaVersion > const_1.maxSchemaVersion || schemaVersion < 0) {
    console.log("Schema version must be between 0 and ", const_1.maxSchemaVersion);
    process.exit();
}
if (options.verbose) {
    console.info("Connecting to", url);
}
const logger = new tslog_1.Logger({ minLevel: options.verbose ? "silly" : "info", displayDateTime: false, displayFunctionName: false, displayLogLevel: false, displayFilePath: "hidden" });
const socket = new ws_1.WebSocket(url, { handshakeTimeout: options.timeout * 1000 });
socket.on("open", function open() {
    socket.send(JSON.stringify({
        messageId: "api-schema-id",
        command: "set_api_schema",
        schemaVersion: schemaVersion,
    }));
    socket.send(JSON.stringify({
        messageId: "start-listening-result",
        command: "start_listening",
    }));
});
let rl;
let silendCommandTimeout;
socket.on("message", (data) => {
    const msg = JSON.parse(data.toString());
    if (msg.type === "event") {
        const event = msg;
        if (event.event.source === "server" && event.event.event === "shutdown") {
            handleShutdown(options.command === undefined ? 0 : 2);
        }
        else if (options.command !== undefined && event.event.event === "command result") {
            const source = options.command.split(".")[0];
            const command = options.command.split(".")[1];
            if (source === "device") {
                const deviceEvent = event.event;
                if (deviceEvent.source === source && deviceEvent.command === command) {
                    if (silendCommandTimeout) {
                        clearTimeout(silendCommandTimeout);
                        silendCommandTimeout = undefined;
                    }
                    handleShutdown(deviceEvent.returnCode === 0 ? 0 : 2);
                }
            }
            else if (source === "station") {
                const stationEvent = event.event;
                if (stationEvent.source === source && stationEvent.command === command) {
                    if (silendCommandTimeout) {
                        clearTimeout(silendCommandTimeout);
                        silendCommandTimeout = undefined;
                    }
                    handleShutdown(stationEvent.returnCode === 0 ? 0 : 2);
                }
            }
        }
    }
    else if (msg.type === "result") {
        const resultMsg = msg;
        switch (resultMsg.messageId) {
            case "start-listening-result":
                {
                    if (options.command === undefined) {
                        rl = readline.createInterface({
                            input: process.stdin,
                            output: process.stdout,
                            terminal: true,
                            prompt: c.cyan.bold("eufy-security> "),
                            completer: (line) => {
                                const hits = commands.filter((c) => c.startsWith(line) && line.substring(c.length) === "");
                                return [hits, line];
                            }
                        });
                        rl.prompt(true);
                        rl.on("line", (line) => {
                            cmd(line.split(" ")).then((args) => {
                                switch (args[0]) {
                                    case "exit":
                                    case "quit":
                                        rl.close();
                                        break;
                                }
                                rl.prompt(true);
                            }).catch();
                        }).on("close", () => {
                            handleShutdown();
                        });
                    }
                    else {
                        cmd([options.command].concat(options.arguments !== undefined ? options.arguments : []), options.command !== undefined);
                    }
                    break;
                }
            case `internal_${command_2.DeviceCommand.getPropertiesMetadata}`: {
                const result = resultMsg.result;
                if (result.serialNumber !== undefined) {
                    devicePropertiesMetadata[result.serialNumber] = result.properties;
                    emitter.emit(`device.get_properties_metadata.${result.serialNumber}`);
                }
                break;
            }
            case `internal_${command_3.StationCommand.getPropertiesMetadata}`:
                const result = resultMsg.result;
                if (result.serialNumber !== undefined) {
                    stationPropertiesMetadata[result.serialNumber] = result.properties;
                    emitter.emit(`station.get_properties_metadata.${result.serialNumber}`);
                }
                break;
            case (0, utils_1.convertCamelCaseToSnakeCase)(options.command?.split(".")[1]):
                if (resultMsg.success) {
                    const msgSuccess = msg;
                    // eslint-disable-next-line @typescript-eslint/no-explicit-any
                    if (msgSuccess.result && msgSuccess.result.async === undefined) {
                        logger.info(msgSuccess.result);
                        handleShutdown(msg.success ? 0 : 2);
                    }
                    else {
                        silendCommandTimeout = setTimeout(() => {
                            handleShutdown(3);
                        }, options.timeout * 1000);
                    }
                }
                else {
                    handleShutdown(msg.success ? 0 : 2);
                }
                break;
        }
    }
    if ((msg.type === "result" && !msg.messageId.startsWith("internal_")) || msg.type !== "result") {
        if (options.command === undefined) {
            if (options.verbose) {
                if (rl)
                    console.log();
                logger.info("Response:", msg);
            }
            else {
                if (rl)
                    console.log();
                console.dir(msg, { depth: 10 });
            }
            rl?.prompt(true);
        }
        else if (options.verbose) {
            logger.info("Response:", msg);
        }
    }
});
socket.on("error", (error) => {
    if (options.verbose) {
        console.log(error);
    }
    handleShutdown(4);
});
let closing = false;
const handleShutdown = (exitcode = 0) => {
    if (silendCommandTimeout) {
        clearTimeout(silendCommandTimeout);
        silendCommandTimeout = undefined;
    }
    // Pressing ctrl+c twice.
    if (closing) {
        process.exit();
    }
    // Close gracefully
    closing = true;
    if (options.verbose) {
        console.log("Shutting down");
    }
    socket.close(1000);
    process.exit(exitcode);
};
process.on("SIGINT", handleShutdown);
process.on("SIGTERM", handleShutdown);
//# sourceMappingURL=client.js.map